@@ -23,15 +23,23 @@ Via a template such as:
 	[% USE Java %]
 	[% Use JavaSQL %]
 	...
-	mySelect	= myConnection.prepareStatement(
-	"select [% JavaSQL.columnNames.join(", ") %] from [% JavaSQL.tables.join(", ") %]
-	[%- IF JavaSQL.where -%]
-	 where [% JavaSQL.where %] 
+	String query =
+	"select [% JavaSQL.columnNames.join(", ") %] from [% JavaSQL.tables.join(", ") %]";
+	[% IF JavaSQL.where %]
+	query += " where [% JavaSQL.where %] ";
+	[% ELSE %]
+	query += " where 0=0 ";
 	[% END %]
-	[%- IF JavaSQL.order -%]
-	order by [% JavaSQL.order %]
-	[% END %]"
-	);
+	[% FOREACH JavaSQL.columns %]
+	if (${varName}Set) { query += " and $name = ?"; }
+	[% END %]
+	[% IF JavaSQL.order %]
+	query += "order by [% JavaSQL.order %]";
+	[% END %]
+	stmt = myConnection.prepareStatement(query);
+
+Just use the DBClass.template from the distribution for fully functional
+database classes.
 
 =head1 DESCRIPTION
 
@@ -49,6 +57,7 @@ use strict;
 use base qw(Template::Plugin);
 use Carp;
 use DBI;
+use Template::Plugin::Java::Constants qw/:all/;
 use Template::Plugin::Java::Utils qw/parseOptions sqlType2JavaType/;
 
 use constant QUERY => qr/
@@ -64,7 +73,7 @@ use constant QUERY => qr/
 	\s*$)
 /xi;
 
-my $dbh = DBI->connect (
+my $dbh = DBI->connect_cached (
 	$ENV{DBI_DSN},
 	$ENV{DBI_USER},
 	$ENV{DBI_PASS},
@@ -96,7 +105,7 @@ sub new {
 	}
 
 # Now process the sql context if any.
-	my $vars = $self->{context}->stash->get('variables');
+	my $vars = $self->context->stash->get('variables');
 	my $spec = delete $vars->{'sql:'};
 
 	return unless $spec;
@@ -119,20 +128,47 @@ sub new {
 			map { $dbh->type_info($_)->{TYPE_NAME} }
 				@{ $sth->{TYPE} };
 	
-	$self->{sth}     = $sth;
+	$sth->finish;
+
 	$self->{columns} = [sort @$columns];
 	$self->parseQuery($query);
 
 	$result = parseOptions($result);
 
 	@{$self->{column2var}}{ sort @$columns } = sort keys %$result;
+	@{$self->{var2column}}{ sort keys %$result } = sort @$columns;
 
 # Place the variable info from the table back into main variables hash.
 	@$vars{ keys %$result } = values %$result;
 
+# If only one table used in query, check whether all columns are used (this
+# makes it updatable in the underlying templates). Ideally, we should be
+# checking if all the primary keys are being used.
+#
+# Also, this can be overridden in the context or constructor, which is not a
+# good idea.
+	if (!exists $self->{updatable} && $self->tableCount == 1) {
+		my $test_sth = $dbh->prepare_cached (
+			"select * from ".$self->tables->[0]
+		);
+
+		if ($test_sth->{NUM_OF_FIELDS} == $sth->{NUM_OF_FIELDS}) {
+			$self->{updatable} = TRUE;
+		}
+	} elsif (exists $spec->{updatable}) {
+		$self->{updatable} = $spec->{updatable};
+	}
+
 	return $self;
 }
 
+=item B<context>
+
+Returns Template::Context this instance has, if any.
+
+=cut
+sub context { $_[0]->{context} }
+
 =item B<query>
 
 The complete query, either supplied at instantiation or inferred from other
@@ -162,6 +198,13 @@ Number of tables used by query.
 =cut
 sub tableCount { scalar @{$_[0]->{tables}} }
 
+=item B<updatable>
+
+Whether the query used can be used to update the table.
+
+=cut
+sub updatable { $_[0]->{updatable} }
+
 =item B<order>
 
 An ORDER BY clause, if one was used.
@@ -176,6 +219,38 @@ A list of column names used in the query.
 =cut
 sub columnNames { $_[0]->{columns} }
 
+=item B<isColumn>
+
+Check whether a variable name is a column.
+
+=cut
+sub isColumn {
+	my ($self, $name) = @_;
+	$name ||= $self->{context}->stash->get('name');
+	  
+	scalar grep { $name eq $_ } @{$_[0]->{columns}}
+		and return TRUE;
+	scalar grep { $self->varToColumn($name) eq $_ }
+					@{$_[0]->{columns}}
+		and return TRUE;
+
+	return FALSE;
+}
+
+=item B<varToColumn>
+
+Gives the column name mapping of the Java variable name.
+
+=cut
+sub varToColumn { return $_[0]->{var2column}{$_[1]} }
+
+=item B<columnToVar>
+
+Gives the Java variable name mapping of the column name.
+
+=cut
+sub columnToVar { return $_[0]->{column2var}{$_[1]} }
+
 =item B<columnCount>
 
 Number of columns returned from query.
@@ -191,7 +266,7 @@ See L</SYNOPSYS>.
 =cut
 sub columns {
 	my $self    = shift;
-	my $vars    = $self->{context}->stash->get('variables');
+	my $vars    = $self->context->stash->get('variables');
 
 	my $position= 1;
 
